#include <stdio.h>
#include "va108xx.h"
#include "reb_log.h"
#include "reb_board.h"
#include "driver_common.h"
#include "reb_timer.h"
#include "gpio_va108xx.h"


#define TIM_FUNC_01   0x1

#define TIM_STATUS_SEL_1  0x1  
#define TIM_STATUS_SEL_2  0x2  
#define TIM_STATUS_SEL_3  0x3  
#define TIM_STATUS_SEL_4  0x4  
#define TIM_STATUS_SEL_5  0x5  
#define TIM_STATUS_SEL_6  0x6  

/********************************
*
*
*
**********************************/
int CONFIG_TIM0(void)
{
	
	    VOR_GPIO->BANK[0].IRQ_ENB = 1<< 12   ;  // set PA[12] Enable bit 
	
	
			VOR_TIM0->RST_VALUE = 0x99 ; 
	    VOR_TIM0->CNT_VALUE = 0x88   ; 
	    VOR_TIM0->ENABLE = 0x88   ;  
	    VOR_TIM0->CSD_CTRL = 0x88   ;  
			VOR_TIM0->CASCADE0 = 0x99 ; 
			VOR_TIM0->CASCADE1 = 0x99 ; 
			VOR_TIM0->CASCADE2 = 0x99 ; 
	    VOR_TIM0->PWMA_VALUE = 0x88   ;  
	    VOR_TIM0->PWMB_VALUE = 0x88   ; 	
	    VOR_TIM0-> CTRL = 0x88  ;  
	    return(0)  ;  
}	

/********************************
*  Input capture function - Uses timer to measure time a button is pressed
*   a) configure port pin as input and enable interrupt
*   b) configure TIM channel as input 
**********************************/

#define  FUNCSEL1 0x1     //  
#define T1_rollover 0xFF00000
#define IRQ_PA11_IRQn OC31_IRQn  
#define IRQ_PA11_IRQ_PRIORITY  0 
//****************************************************************
// *  Routine to setup TIM1 to count whenever PA[11] is low  (Pa11 is the push button on the REB1 board
//*  - A pin interrupt will be generated for falling and rising edges
//*  - PA[11] will have the input filter enabled and uses CLKDIV1
//*  - TIM1 will be setup to start counting on a falling edge int of PA[11]
//*  - inside the PA11 ISR, the TIM1 cascade 2 will be setup to shut the TIM down when a falling edge int on PA11 occurs
//*****************************************************************
uint32_t CONFIG_TIM1(void)
{
		  uint32_t temp;
	
			VOR_SYSCONFIG->TIM_CLK_ENABLE |= 0xFFFF  ;  //  enable clocks to lower 16 TIM blocks
	
	//  ***  setup condition for cascade 0 input - PA11 edge(s)  			
			VOR_SYSCONFIG->IOCONFIG_CLKDIV1 = 0xff  ;  //  set clkdiv0 to div256
	
		  VOR_IOCONFIG->PORTA[11]= ((5<<IOCONFIG_PORTA_FLTTYPE_Pos) | (1<<IOCONFIG_PORTA_FLTCLK_Pos)
                         	| IOCONFIG_PORTA_PLEVEL_Msk | IOCONFIG_PORTA_PEN_Msk );
	
			VOR_GPIO->BANK[0].DIR &= ~(1 << 11);       // set PA[11] to an input
			VOR_GPIO->BANK[0].IRQ_SEN &= ~( 1 << 11);  // set SEN for PA11 to 0 to stat gen from signal transition
			VOR_GPIO->BANK[0].IRQ_EDGE |= 1 << 11   ;  //  set EDGE for PA11 to 1 to gen stat for either edge
			
		  VOR_IRQSEL->PORTA[11]  = IRQ_PA11_IRQn;    // IRQSEL redirects PA to NVIC IRQ input 31 
	
			VOR_GPIO->BANK[0].DIR |= (1 << 1);    // set PA[1] to an output
			VOR_IOCONFIG->PORTA[1]= (FUNCSEL1 << IOCONFIG_PORTA_FUNSEL_Pos); // set pin func to TIM1

			for(temp=0; temp < 1000 ; temp ++)   
					{ 
						  ; 
					}    // Wait for PA[11] pin and filter to settle 
			
 	    VOR_TIM1->CTRL =  ( TIM_STATUS_SEL_1<< TIM1_CTRL_STATUS_SEL_Pos) ;  // Output when active for debug purposes only 
			VOR_TIM1->RST_VALUE = T1_rollover ; 
	    VOR_TIM1->CNT_VALUE = T1_rollover ; 
	
      VOR_TIM1->CSD_CTRL |= ( TIM1_CSD_CTRL_CSDEN0_Msk   ) ;
	                 // Setup cascade control so timer counts PORTA[11] transitions low
			VOR_TIM1->CASCADE0 = (TIM_CAS_SRC_PORTA_11) ;   // set cascade 0 source to PORTA[11]
			VOR_TIM1->CASCADE1 = 0x00 ; 
			VOR_TIM1->CASCADE2 = (TIM_CAS_SRC_PORTA_11) ;   // set cascade 0 source to PORTA[11]	

			VOR_TIM1->ENABLE = 0x1   ;    // enable TIM1
					
			NVIC_SetPriority(IRQ_PA11_IRQn,IRQ_PA11_IRQ_PRIORITY);
			NVIC_EnableIRQ(IRQ_PA11_IRQn);
 
		  VOR_GPIO->BANK[0].IRQ_ENB |= 1 << 11   ;  // set PA[11] Enable bit 
					
			return(temp)  ;  
}	

//****************************************************************
//*  Routine to setup TIM3 to count rising edges on PA[8]  
//*      - PA8 is setup to generate an interrupt for every falling edge
//*      - TIM3 is setup to use cascade 0 (PA9) for gating the counter 
//*     (Note:  For this program to run, a jumper must be placed between PA[8] and PA[1]. 
//*****************************************************************
#define T3_rollover 0x20000
uint32_t CONFIG_TIM3(void)
{
		  uint32_t temp ;
 
	    VOR_TIM3->CTRL = (TIM1_CTRL_IRQ_ENB_Msk | ( 2 << TIM1_CTRL_STATUS_SEL_Pos))  ;  //  
			VOR_TIM3->RST_VALUE = T3_rollover ; 
	    VOR_TIM3->CNT_VALUE = T3_rollover ; 
	
      VOR_TIM3->CSD_CTRL |= ( TIM1_CSD_CTRL_CSDEN0_Msk ) ;
	                 // Setup cascade control so timer only counts when input is low
			VOR_TIM3->CASCADE0 = (TIM_CAS_SRC_PORTA_8) ;   // set cascade 0 source to PORTA11
			VOR_TIM3->CASCADE1 = 0x00 ; 
			VOR_TIM3->CASCADE2 = 0x00 ; 	
			VOR_TIM3->ENABLE = 0x1   ;    // enable TIM1
			
//  ***  setup condition for cascade 0 input  
    	VOR_SYSCONFIG->IOCONFIG_CLKDIV1 = 0x80  ;  //  set clkdiv0 to div128
			
			VOR_IOCONFIG->PORTA[1]= IOCONFIG_PORTA_PLEVEL_Msk | IOCONFIG_PORTA_PEN_Msk |(FUNCSEL1 << IOCONFIG_PORTA_FUNSEL_Pos); // set pin func to TIM1
			VOR_IOCONFIG->PORTA[8]= ((5<<IOCONFIG_PORTA_FLTTYPE_Pos) | (1<<IOCONFIG_PORTA_FLTCLK_Pos) | IOCONFIG_PORTA_PLEVEL_Msk | IOCONFIG_PORTA_PEN_Msk );

      VOR_GPIO->BANK[0].IRQ_EVT &= ~(1 << TIM_CAS_SRC_PORTA_8)   ;  // clr EVT -> STAt generated when  H->L transition in IRQ_SEN transition mode 
			VOR_GPIO->BANK[0].IRQ_SEN &= ~(1 << TIM_CAS_SRC_PORTA_8)   ;  // clr SEN  ->  Status is generated from signal transition
			VOR_GPIO->BANK[0].IRQ_EDGE &= ~(1 << TIM_CAS_SRC_PORTA_8)  ;  // clr EDGE -> Status is generated from signal transition as configured by IRQ_EVT bits 
			VOR_GPIO->BANK[0].IRQ_ENB |= 1 << TIM_CAS_SRC_PORTA_8   ;      // set PA[8] IRQ Enable bit 
		 
			return(temp)  ;  
}	

//*************************************************************************
//  Routine to configure TIM23 as 10 msec TICK generator  
//      - includes IRQSEL and NVIC configuration 
//**************************************************************************
/**
	\brief IRQ30 services TIM23  (10 msec Tic)   
*/ 
#define IRQ_TIM1_PRIORITY  0
#define IRQ_TIM1_INDEX     1
#define IRQ_TIM1_IRQn     OC1_IRQn
#define IRQ_TIM23_IRQn     OC30_IRQn  
#define IRQ_TIM23_IRQ_PRIORITY  0  

uint32_t CONFIG_TIM23_periodic_int(void)
{
			VOR_SYSCONFIG->TIM_CLK_ENABLE |= (1 << 23);  //  enable clock to TIM23 block 
	
		  VOR_TIM23->CTRL = (TIM23_CTRL_IRQ_ENB_Msk | ( 0x01 << TIM1_CTRL_STATUS_SEL_Pos))  ;  //  
			VOR_TIM23->RST_VALUE = (100e-3 * 50e6)-1 ;   // setup rst value to 500 msec
	    VOR_TIM23->CNT_VALUE = (100e-3 * 50e6)-1 ; 
	
			  VOR_TIM23->ENABLE = 0x1   ;    // enable TIM23
	
	      VOR_IRQSEL->TIM[23]  = IRQ_TIM23_IRQn;   // IRQSEL redirects TIM23 to NVIC IRQ input 30 
				NVIC_SetPriority(IRQ_TIM23_IRQn,IRQ_TIM23_IRQ_PRIORITY);
				NVIC_EnableIRQ(IRQ_TIM23_IRQn);	
   
				SysTick->CTRL  = SysTick_CTRL_CLKSOURCE_Msk |
                   SysTick_CTRL_ENABLE_Msk; 

       return(0)  ;   	
}

//  ****************************************************
//  Global variable declarations  
// *****************************************************
     uint32_t  cnt_10ms,cnt_button_presses    ;    //  global variable for keeping track of 10 msec  
     uint32_t  old_SYSTICK    ;    //  global variable for debuging 10 msec 
     uint32_t  new_SYSTICK    ;    //  global variable for keeping track of 10 msec 
     uint32_t  PA8_pulse_cnt_array[16],  i = 0  ;  
		 uint32_t  input_pulse[16], k ;  

	
//*****************************************************
//  Interrupt Subroutine for TIM23 (NVIC IRQ30)  
//   - calculate pulse count 
//   - toggle PA[4] to show interval 
//*****************************************************
void OC30_IRQHandlerX(void)  
{
 	uint32_t elapsed_time  ;
	
	cnt_10ms ++  ; 
	new_SYSTICK = SysTick->VAL  ;    //     
	elapsed_time = old_SYSTICK - new_SYSTICK   ;  
	old_SYSTICK = new_SYSTICK  ;  

	if(i>16)  	{ i = 0  ;  }
	
	PA8_pulse_cnt_array[i++] = (VOR_TIM3->RST_VALUE - VOR_TIM3->CNT_VALUE)  ;  // store pulse count in 16 unit array
	     VOR_TIM3->CNT_VALUE = VOR_TIM3->RST_VALUE  ;  
		
	 		VOR_GPIO->BANK[0].DIR |= (1 << 4);    // set PA[4] to an output 
	    VOR_GPIO->BANK[0].TOGOUT |= (1 << 4);  // Toggle PA[4] to show interval of interrupt
}
//*****************************************************
//  Interrupt Subroutine for PA11 (NVIC IRQ31)  
//   - increment button_press count  
//   - if (falling edge cause int), then 
//          a) setup cascade control 2 to shutdown on rising edge
//   - else ( if rising edge caused int), then 
//          a) setup cascade control 0 to start
//          b)  calculate pulse width and 
//*****************************************************
void OC31_IRQHandlerX(void)  
{
 	uint32_t elapsed_time  ;
	
	cnt_button_presses  ++  ; 
	new_SYSTICK = SysTick->VAL  ;    //     
	elapsed_time = old_SYSTICK - new_SYSTICK   ;  
	old_SYSTICK = new_SYSTICK  ;  
	
	if(VOR_TIM1->CSD_CTRL &  TIM1_CSD_CTRL_CSDEN0_Msk )    // if yes, this was falling edge
	{
		VOR_TIM1->CSD_CTRL = TIM1_CSD_CTRL_CSDEN2_Msk  ;  //  setup PA11 next edge to stop TIM1
		     if((VOR_GPIO->BANK[0].DATAIN & (1<<11)) ) 
				 { VOR_TIM1->CSD_CTRL = TIM1_CSD_CTRL_CSDEN0_Msk; } //  precautionary measure if ever rise/fall edges get out of sync 
	}
	else
	{
 		VOR_TIM1->CSD_CTRL = TIM1_CSD_CTRL_CSDEN0_Msk   ;  //  setup PA11 next edge to start TIM1
		VOR_TIM1->ENABLE = 0x1   ;    // enable TIM1
		input_pulse[k] = (VOR_TIM1->RST_VALUE - VOR_TIM1->CNT_VALUE)  ;  // calc pulse width and place in array 
		k++  ;  
		VOR_TIM1->CNT_VALUE = VOR_TIM1->RST_VALUE  ;
		if(k>16)  	{ k = 0  ;  }	
	}
}



//*********************************************************************
//  ** routine to pull PA9 low  for a time dependent on the input variable  
//**********************************************************************

uint32_t  pulse_PA9_low(uint32_t count)
{
			uint32_t i  ; 
			i = count ;  
			VOR_GPIO->BANK[0].DATAOUT |= (1 << 9); // set PA[9]  output to 1 
	 		VOR_GPIO->BANK[0].DIR |= (1 << 9);    // set PA[9] to an output 
	
	    VOR_GPIO->BANK[0].DATAOUT &= ~(1 << 9);    // set PA[9]  output to 0 
	    for(i=0;i<count;i++)  { VOR_GPIO->BANK[0].DATAOUT &= ~(1 << 9);  }  
	
	    VOR_GPIO->BANK[0].DATAOUT |= (1 << 9);    // set PA[9]  output to 1 
			return(0)  ;	 
}

/***************************************************************************************
 * @file     reb_main.c
 * @version  V1.0
 * @date     31. March 2016
 *
 * @note
 * VORAGO Technologies 
 *
 * @note
 * Copyright (c) 2013-2016 VORAGO Technologies. 
 *
 * @par
 * BY DOWNLOADING, INSTALLING OR USING THIS SOFTWARE, YOU AGREE TO BE BOUND BY 
 * ALL THE TERMS AND CONDITIONS OF THE VORAGO TECHNOLOGIES END USER LICENSE AGREEMENT. 
 * THIS SOFTWARE IS PROVIDED "AS IS". NO WARRANTIES, WHETHER EXPRESS, IMPLIED
 * OR STATUTORY, INCLUDING, BUT NOT LIMITED TO, IMPLIED WARRANTIES OF MERCHANTABILITY 
 * AND FITNESS FOR A PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. VORAGO TECHNOLOGIES 
 * SHALL NOT, IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL, OR CONSEQUENTIAL 
 * DAMAGES, FOR ANY REASON WHATSOEVER.
 *
 ****************************************************************************************/


int main()
{
	uint32_t i , pulse[16];  
	VOR_TIM_SIGNAL s0, s8; 
	
	/* Enable clocks for all peripherals */ 
	VOR_SYSCONFIG->PERIPHERAL_CLK_ENABLE = ( CLK_ENABLE_PORTA | CLK_ENABLE_PORTB | CLK_ENABLE_SPIA | CLK_ENABLE_SPIB | CLK_ENABLE_SPIC |
																					 CLK_ENABLE_UARTA | CLK_ENABLE_UARTB | CLK_ENABLE_I2CA | CLK_ENABLE_I2CB | CLK_ENABLE_IRQSEL |
                                           CLK_ENABLE_IOMGR | CLK_ENABLE_UTILITY | CLK_ENABLE_PORTIO | CLK_ENABLE_SYSTEM ); 
  VOR_SYSCONFIG->TIM_CLK_ENABLE = 0xFFFFFF  ;  //  enable clocks to all 24 TIM blocks 
	
	i = CONFIG_TIM1()  ;  
	i = CONFIG_TIM3()  ;  
  i = CONFIG_TIM23_periodic_int() ;  
	
	while(1)
	{
		
//			   pulse_PA9_low(1000)  ;  
	}
	
	/* Initialize Logging module */ 
	VOR_Log_Init(RTT, LOG_DBG); 
	VOR_Log(LOG_INFO, ANSI_COLOR_GREEN"Welcome to Vorago Technologies REB1 VA108xx software! \n");
	if( VOR_SYSCONFIG->PROCID == PROC_ID_TORCH )
		VOR_Log(LOG_INFO, ANSI_COLOR_GREEN"VA10820 MCU ...\n");
	else
		VOR_Log(LOG_INFO, ANSI_COLOR_GREEN"VA10800 MCU ... \n");
	VOR_Log(LOG_INFO, "Restarted... \n");
	VOR_Log(LOG_INFO, "Reset Status: 0x%08x \n", VOR_SYSCONFIG->RST_STAT); 
	
	/*__________________________________________________
	 |Enable Timer|PULSE Timers| PWM PULSE | PORT PIN # |
	 |____________|____________|___________|____________|
	 | Timer 0    |  Timer 1   | Timer 7   |		PA[7]   |
	 |            |  Timer 2   | 					 |  	PA[2]	  |
	 |	   	      |	 Timer 3   | Timer 8   | 		PA[8]   |
	 | 				    |	 Timer 4   |           |		PA[4]   | 
	 |				    |	 Timer 5   | Timer 9   |  	PA[9]   |
	 |				    |	 Timer 6   |           |		PA[6]   |
	 |____________|____________|___________|____________|
	*/ 							
	VOR_TIM_Initialize(); 
	/*				
	sector		|	 0	  |	 1	  |	 2	  |  3    |  4    |  5    |
	    
						 ________________________________________________
		S0			|       |       |       |       |       |       | 
					__|       |       |       |       |       |       |
						|       |       |       |       |       |       |  
	
														 _______________ 
	  S3			|       |       |       |       |       |       | 
					__|_______|_______|				|				|_______|_______|      
						|       |       |       |       |       |       |  
	 
						|_   _  |_   _  |_   _  |_   _   _   _  |_   _  | 
		S8 			| | | | | | | | | | | | | | | | | | | | | | | | | 
					 _| |_| |_| |_| |_| |_| |_| |_| |_| |_| |_| |_| |_|_
					  |       |       |       |       |       |       |
	 
						|       |       |       |_   _  |_   _  |       |
		S8 			|       |       |       | | | | | | | | |       | 
					__|_______|_______|_______| |_| |_| |_| |_|_______|_
						|       |       |       |       |       |       |
   */	
	s8.trig = (VOR_TIM_TRIGSIGNAL){.timer_instance = 0, .port_num = 0, .pin_num = 0, .cycle = 18000}; 
	s8.pulse = (VOR_TIM_PULSESIGNAL){.timer_instance = 3, .port_num = 0, .pin_num = 3, .cycle = 18000, .start_offset = 9000, .end_offset = 15000, .inverse = false}; 
	s8.pwmpulse = (VOR_TIM_PWMPULSESIGNAL){.timer_instance = 8, .port_num = 0, .pin_num = 8, .cycle = 50}; 
	s8.type = SIGNAL_TYPE_PWMPULSE; 
	VOR_TIM_Create(s8); 		
		
	s0.trig = (VOR_TIM_TRIGSIGNAL){.cycle = 18000, .pin_num = 0, .port_num = 0, .timer_instance = 0}; 
	s0.type = SIGNAL_TYPE_TRIGGER; 	 
	VOR_TIM_Create(s0); 
	
	VOR_TIM_Enable(); 
	
	for(i=0;i<16;i++)  { pulse[i]= 0x00  ;  }    // clear the pulse array  
	 
	i = pulse_PA9_low(100)  ;   

	  
	while(1) 
	{
			if((VOR_GPIO->BANK[0].DATAIN & (1<<2)) !=0)    //  if the port pin is high, check the timer for a non-reset value
			{
 				if(VOR_TIM1->CNT_VALUE <= (T1_rollover-50) ) 
				{
					pulse[i++] = T1_rollover - VOR_TIM1->CNT_VALUE  ; 
					VOR_TIM1->CNT_VALUE = T1_rollover  ;  
					if(i==16)  {  i = 0 ;  } 
				}
			}	
	   pulse_PA9_low(1000)  ;  
	}
}
